" Vim filetype plugin
" Language:	Brighterscript
" Maintainer:	Riley Bruins <ribru17@gmail.com>
" Last Change:	2025 Jun 17

if exists('b:did_ftplugin')
  finish
endif
let b:did_ftplugin = 1

runtime! ftplugin/brightscript.vim
